## Properties of the Lac operon model
## specified in CTL
## written as an input for NuSMV
## developed for the paper
## “Modular Verification of Interactive Systems with an Application to Biology”.
## by P. Drabik, A.Maggiolo-Schettini and P. Milazzo
## Universita di Pisa, Italy
## version from 18/12/2010
################################################################################

## initialisation of the model
go

## adding all the properties to the property database

## reachability of all the states of all the modules
add_property -c "EF state1 IN allo"
add_property -c "EF state2 IN allo"
add_property -c "EF state3 IN allo"
add_property -c "EF state4 IN allo"
add_property -c "EF state5 IN allo"
add_property -c "EF state1 IN beta"
add_property -c "EF state2 IN beta"
add_property -c "EF state3 IN beta"
add_property -c "EF state4 IN beta"
add_property -c "EF state5 IN beta"
add_property -c "EF state6 IN beta"
add_property -c "EF state7 IN beta"
add_property -c "EF state8 IN beta"
add_property -c "EF state9 IN beta"
add_property -c "EF state01 IN pos"
add_property -c "EF state02 IN pos"
add_property -c "EF state03 IN pos"
add_property -c "EF state04 IN pos"
add_property -c "EF state05 IN pos"
add_property -c "EF state06 IN pos"
add_property -c "EF state07 IN pos"
add_property -c "EF state08 IN pos"
add_property -c "EF state09 IN pos"
add_property -c "EF state10 IN pos"
add_property -c "EF state11 IN pos"
add_property -c "EF state12 IN pos"
add_property -c "EF state1 IN glu"
add_property -c "EF state2 IN glu"
add_property -c "EF state3 IN glu"
add_property -c "EF state4 IN glu"
add_property -c "EF state5 IN glu"
add_property -c "EF state6 IN glu"
add_property -c "EF state7 IN glu"
add_property -c "EF state8 IN glu"
add_property -c "EF state1 IN lac"
add_property -c "EF state2 IN lac"
add_property -c "EF state3 IN lac"
add_property -c "EF state4 IN lac"
add_property -c "EF state5 IN lac"
add_property -c "EF state6 IN lac"
add_property -c "EF state01 IN op"
add_property -c "EF state02 IN op"
add_property -c "EF state03 IN op"
add_property -c "EF state04 IN op"
add_property -c "EF state05 IN op"
add_property -c "EF state06 IN op"
add_property -c "EF state07 IN op"
add_property -c "EF state08 IN op"
add_property -c "EF state09 IN op"
add_property -c "EF state10 IN op"
add_property -c "EF state11 IN op"
add_property -c "EF state12 IN op"
add_property -c "EF state01 IN repr"
add_property -c "EF state02 IN repr"
add_property -c "EF state03 IN repr"
add_property -c "EF state04 IN repr"
add_property -c "EF state05 IN repr"
add_property -c "EF state06 IN repr"
add_property -c "EF state07 IN repr"
add_property -c "EF state08 IN repr"
add_property -c "EF state09 IN repr"
add_property -c "EF state10 IN repr"
add_property -c "EF state11 IN repr"
add_property -c "EF state12 IN repr"

## properties demonstrating that the fairness is satisfied in each module
add_property -c "AG(allo.toTrue_allo -> AF !allo.toTrue_allo)"
add_property -c "AG(beta.toTrue_beta -> AF !beta.toTrue_beta)"
add_property -c "AG( pos.toTrue_pos  -> AF ! pos.toTrue_pos )"
add_property -c "AG( glu.toTrue_glu  -> AF ! glu.toTrue_glu )"
add_property -c "AG( lac.toTrue_lac  -> AF ! lac.toTrue_lac )"
add_property -c "AG(  op.toTrue_op   -> AF !  op.toTrue_op  )"
add_property -c "AG(repr.toTrue_repr -> AF !repr.toTrue_repr)"

## sample properties
# the verification of a true property results in a passing of the check
#add_property -c "AG(repr.b1 -> (repr.b2|repr.b3) )"
# the verification of a false property results in a counterexample trace
#add_property -c "AG(repr.b1 -> (repr.b2) )"
# a true property considering two modules
#add_property -c "AF(notToTrue -> (allo.allo_low & beta.beta_low))"
# a false property considering two modules
#add_property -c "AG(notToTrue -> (allo.allo_low & beta.beta_low))"


## properties of Main1 - representing the whole sync-program P^{I}
## from the paper
# The allolactose bound to the repressor implies that the operon is repressed
add_property -c "AG(notToTrue -> (repr.ballo -> op.rep))"

# The increase of allolactose concentration can only be mediated
# by β-galactosidase in low concentration
add_property -c "AG(notToTrue -> (allo.allo_none & beta.beta_high -> A[!allo.allo_low U beta.beta_low]))"

# The operon will necessarily oscillate between repressed and unrepressed state
add_property -c "AG((op.rep -> AF(!op.rep)) & (!op.rep -> AF(op.rep)))"

# When the glucose concentration drops, the lac operon will eventually be fully expressed
add_property -c "AG(glu.glu_low -> AF(op.act & !op.rep))"

# When glucose con- centration is low and CRP−cAMP complex is bound to the operon, 
# the lac operon will eventually be fully expressed
add_property -c "(pos.crp-camp & glu.glu_low -> AF(op.act & !op.rep))"

################################################################################
## properties of projections

## properties of Main2 - projection onto {op,repr}
# add_property -c "AG(notToTrue -> (repr.ballo -> op.rep))"
# add_property -c "AG((op.rep -> AF(!op.rep)) & (!op.rep -> AF(op.rep)))"

## properties of Main3 - projection onto {allo,beta}
# add_property -c "AG(notToTrue -> (allo.allo_none & beta.beta_high -> A[!allo.allo_low U beta.beta_low]))"

## properties of Main4 - projection onto {op}
# add_property -c "AG((op.rep -> AF(!op.rep)) & (!op.rep -> AF(op.rep)))"

## properties of Main5 - projection onto {beta,pos,glu,op}
# add_property -c "AG(glu.glu_low -> AF(op.act & !op.rep))"
# add_property -c "(pos.crp-camp & glu.glu_low -> AF(op.act & !op.rep))"

################################################################################
check_property;
#show_property;
quit;
